@@ -15,12 +15,11 @@ import collections
 
 from oslo_log import log as logging
 from oslo_utils import timeutils
-import six
 
 from heat.common.i18n import _
-from heat.common.i18n import _LE
 from heat.common import param_utils
 from heat.common import template_format
+from heat.common import timeutils as heat_timeutils
 from heat.engine import constraints as constr
 from heat.rpc import api as rpc_api
 
@@ -40,21 +39,16 @@ def extract_args(params):
         try:
             timeout = int(timeout_mins)
         except (ValueError, TypeError):
-            LOG.exception(_LE('Timeout conversion failed'))
+            LOG.exception('Timeout conversion failed')
         else:
             if timeout > 0:
                 kwargs[rpc_api.PARAM_TIMEOUT] = timeout
             else:
                 raise ValueError(_('Invalid timeout value %s') % timeout)
-
-    name = rpc_api.PARAM_DISABLE_ROLLBACK
-    if name in params:
-        disable_rollback = param_utils.extract_bool(name, params[name])
-        kwargs[name] = disable_rollback
-
-    name = rpc_api.PARAM_SHOW_DELETED
-    if name in params:
-        params[name] = param_utils.extract_bool(name, params[name])
+    for name in [rpc_api.PARAM_CONVERGE, rpc_api.PARAM_DISABLE_ROLLBACK]:
+        if name in params:
+            bool_value = param_utils.extract_bool(name, params[name])
+            kwargs[name] = bool_value
 
     adopt_data = params.get(rpc_api.PARAM_ADOPT_STACK_DATA)
     if adopt_data:
@@ -65,12 +59,12 @@ def extract_args(params):
         kwargs[rpc_api.PARAM_ADOPT_STACK_DATA] = adopt_data
 
     tags = params.get(rpc_api.PARAM_TAGS)
-    if tags:
+    if tags is not None:
         if not isinstance(tags, list):
             raise ValueError(_('Invalid tags, not a list: %s') % tags)
 
         for tag in tags:
-            if not isinstance(tag, six.string_types):
+            if not isinstance(tag, str):
                 raise ValueError(_('Invalid tag, "%s" is not a string') % tag)
 
             if len(tag) > 80:
@@ -133,7 +127,7 @@ def translate_filters(params):
         rpc_api.STACK_TIMEOUT: 'timeout',
         rpc_api.STACK_OWNER: 'username',
         rpc_api.STACK_PARENT: 'owner_id',
-        rpc_api.STACK_USER_PROJECT_ID: 'stack_user_project_id',
+        rpc_api.STACK_USER_PROJECT_ID: 'stack_user_project_id'
     }
 
     for key, field in key_map.items():
@@ -170,40 +164,53 @@ def translate_filters(params):
     return params
 
 
-def format_stack_outputs(stack, outputs):
+def format_stack_outputs(outputs, resolve_value=False):
     """Return a representation of the given output template.
 
     Return a representation of the given output template for the given stack
     that matches the API output expectations.
     """
-    def format_stack_output(k):
-        output = {
-            rpc_api.OUTPUT_DESCRIPTION: outputs[k].get('Description',
-                                                       'No description given'),
-            rpc_api.OUTPUT_KEY: k,
-            rpc_api.OUTPUT_VALUE: stack.output(k)
-        }
-        if outputs[k].get('error_msg'):
-            output.update({rpc_api.OUTPUT_ERROR: outputs[k].get('error_msg')})
-        return output
-
-    return [format_stack_output(key) for key in outputs]
+    return [format_stack_output(outputs[key], resolve_value=resolve_value)
+            for key in outputs]
 
 
-def format_stack(stack, preview=False):
+def format_stack_output(output_defn, resolve_value=True):
+    result = {
+        rpc_api.OUTPUT_KEY: output_defn.name,
+        rpc_api.OUTPUT_DESCRIPTION: output_defn.description(),
+    }
+
+    if resolve_value:
+        value = None
+        try:
+            value = output_defn.get_value()
+        except Exception as ex:
+            # We don't need error raising, just adding output_error to
+            # resulting dict.
+            result.update({rpc_api.OUTPUT_ERROR: str(ex)})
+        finally:
+            result.update({rpc_api.OUTPUT_VALUE: value})
+
+    return result
+
+
+def format_stack(stack, preview=False, resolve_outputs=True):
     """Return a representation of the given stack.
 
     Return a representation of the given stack that matches the API output
     expectations.
     """
-    updated_time = stack.updated_time and stack.updated_time.isoformat()
-    created_time = stack.created_time or timeutils.utcnow()
+    updated_time = heat_timeutils.isotime(stack.updated_time)
+    created_time = heat_timeutils.isotime(stack.created_time or
+                                          timeutils.utcnow())
+    deleted_time = heat_timeutils.isotime(stack.deleted_time)
     info = {
         rpc_api.STACK_NAME: stack.name,
         rpc_api.STACK_ID: dict(stack.identifier()),
-        rpc_api.STACK_CREATION_TIME: created_time.isoformat(),
+        rpc_api.STACK_CREATION_TIME: created_time,
         rpc_api.STACK_UPDATED_TIME: updated_time,
-        rpc_api.STACK_NOTIFICATION_TOPICS: [],  # TODO(?) Not implemented yet
+        rpc_api.STACK_DELETION_TIME: deleted_time,
+        rpc_api.STACK_NOTIFICATION_TOPICS: [],  # TODO(therve) Not implemented
         rpc_api.STACK_PARAMETERS: stack.parameters.map(str),
         rpc_api.STACK_DESCRIPTION: stack.t[stack.t.DESCRIPTION],
         rpc_api.STACK_TMPL_DESCRIPTION: stack.t[stack.t.DESCRIPTION],
@@ -225,9 +232,42 @@ def format_stack(stack, preview=False):
         info.update(update_info)
 
     # allow users to view the outputs of stacks
-    if stack.action != stack.DELETE and stack.status != stack.IN_PROGRESS:
-        info[rpc_api.STACK_OUTPUTS] = format_stack_outputs(stack,
-                                                           stack.outputs)
+    if (not (stack.action == stack.DELETE and stack.status == stack.COMPLETE)
+            and resolve_outputs):
+        info[rpc_api.STACK_OUTPUTS] = format_stack_outputs(stack.outputs,
+                                                           resolve_value=True)
+
+    return info
+
+
+def format_stack_db_object(stack):
+    """Return a summary representation of the given stack.
+
+    Given a stack versioned DB object, return a representation of the given
+    stack for a stack listing.
+    """
+    updated_time = heat_timeutils.isotime(stack.updated_at)
+    created_time = heat_timeutils.isotime(stack.created_at)
+    deleted_time = heat_timeutils.isotime(stack.deleted_at)
+
+    tags = None
+    if stack.tags:
+        tags = [t.tag for t in stack.tags]
+    info = {
+        rpc_api.STACK_ID: dict(stack.identifier()),
+        rpc_api.STACK_NAME: stack.name,
+        rpc_api.STACK_DESCRIPTION: '',
+        rpc_api.STACK_ACTION: stack.action,
+        rpc_api.STACK_STATUS: stack.status,
+        rpc_api.STACK_STATUS_DATA: stack.status_reason,
+        rpc_api.STACK_CREATION_TIME: created_time,
+        rpc_api.STACK_UPDATED_TIME: updated_time,
+        rpc_api.STACK_DELETION_TIME: deleted_time,
+        rpc_api.STACK_OWNER: stack.username,
+        rpc_api.STACK_PARENT: stack.owner_id,
+        rpc_api.STACK_USER_PROJECT_ID: stack.stack_user_project_id,
+        rpc_api.STACK_TAGS: tags,
+    }
 
     return info
 
@@ -237,16 +277,19 @@ def format_resource_attributes(resource, with_attr=None):
     if not with_attr:
         with_attr = []
 
+    # Always return live values for consistency
+    resolver.reset_resolved_values()
+
     def resolve(attr, resolver):
         try:
             return resolver._resolver(attr)
         except Exception:
             return None
-    # if 'show' in attribute_schema, will resolve all attributes of resource
+    # if 'show' in attributes_schema, will resolve all attributes of resource
     # including the ones are not represented in response of show API, such as
     # 'console_urls' for nova server, user can view it by taking with_attr
     # parameter
-    if 'show' in six.iterkeys(resolver):
+    if 'show' in resolver:
         show_attr = resolve('show', resolver)
         # check if 'show' resolved to dictionary. so it's not None
         if isinstance(show_attr, collections.Mapping):
@@ -258,7 +301,7 @@ def format_resource_attributes(resource, with_attr=None):
             # remove 'show' attribute if it's None or not a mapping
             # then resolve all attributes manually
             del resolver._attributes['show']
-    attributes = set(list(six.iterkeys(resolver)) + with_attr)
+    attributes = set(resolver) | set(with_attr)
     return dict((attr, resolve(attr, resolver))
                 for attr in attributes)
 
@@ -268,10 +311,12 @@ def format_resource_properties(resource):
         try:
             return resource.properties[prop]
         except (KeyError, ValueError):
+            LOG.exception("Error in fetching property %s of resource %s" %
+                          (prop, resource.name))
             return None
 
     return dict((prop, get_property(prop))
-                for prop in six.iterkeys(resource.properties_schema))
+                for prop in resource.properties_schema.keys())
 
 
 def format_stack_resource(resource, detail=True, with_props=False,
@@ -281,9 +326,9 @@ def format_stack_resource(resource, detail=True, with_props=False,
     Return a representation of the given resource that matches the API output
     expectations.
     """
-    created_time = resource.created_time and resource.created_time.isoformat()
-    last_updated_time = (resource.updated_time and
-                         resource.updated_time.isoformat()) or created_time
+    created_time = heat_timeutils.isotime(resource.created_time)
+    last_updated_time = heat_timeutils.isotime(
+        resource.updated_time or resource.created_time)
     res = {
         rpc_api.RES_UPDATED_TIME: last_updated_time,
         rpc_api.RES_CREATION_TIME: created_time,
@@ -300,8 +345,7 @@ def format_stack_resource(resource, detail=True, with_props=False,
     }
 
     if resource.has_nested():
-        res[rpc_api.RES_NESTED_STACK_ID] = dict(
-            resource.nested().identifier())
+        res[rpc_api.RES_NESTED_STACK_ID] = dict(resource.nested_identifier())
 
     if resource.stack.parent_resource_name:
         res[rpc_api.RES_PARENT_RESOURCE] = resource.stack.parent_resource_name
@@ -309,11 +353,12 @@ def format_stack_resource(resource, detail=True, with_props=False,
     if detail:
         res[rpc_api.RES_DESCRIPTION] = resource.t.description
         res[rpc_api.RES_METADATA] = resource.metadata_get()
-        res[rpc_api.RES_SCHEMA_ATTRIBUTES] = format_resource_attributes(
-            resource, with_attr)
+        if with_attr is not False:
+            res[rpc_api.RES_ATTRIBUTES] = format_resource_attributes(
+                resource, with_attr)
 
     if with_props:
-        res[rpc_api.RES_SCHEMA_PROPERTIES] = format_resource_properties(
+        res[rpc_api.RES_PROPERTIES] = format_resource_properties(
             resource)
 
     return res
@@ -322,7 +367,7 @@ def format_stack_resource(resource, detail=True, with_props=False,
 def format_stack_preview(stack):
     def format_resource(res):
         if isinstance(res, list):
-            return map(format_resource, res)
+            return list(map(format_resource, res))
         return format_stack_resource(res, with_props=True)
 
     fmt_stack = format_stack(stack, preview=True)
@@ -332,23 +377,24 @@ def format_stack_preview(stack):
     return fmt_stack
 
 
-def format_event(event):
-    stack_identifier = event.stack.identifier()
-    event_timestamp = event.timestamp or timeutils.utcnow()
-
+def format_event(event, stack_identifier, root_stack_identifier=None,
+                 include_rsrc_prop_data=True):
     result = {
-        rpc_api.EVENT_ID: dict(event.identifier()),
+        rpc_api.EVENT_ID: dict(event.identifier(stack_identifier)),
         rpc_api.EVENT_STACK_ID: dict(stack_identifier),
         rpc_api.EVENT_STACK_NAME: stack_identifier.stack_name,
-        rpc_api.EVENT_TIMESTAMP: event_timestamp.isoformat(),
+        rpc_api.EVENT_TIMESTAMP: heat_timeutils.isotime(event.created_at),
         rpc_api.EVENT_RES_NAME: event.resource_name,
         rpc_api.EVENT_RES_PHYSICAL_ID: event.physical_resource_id,
-        rpc_api.EVENT_RES_ACTION: event.action,
-        rpc_api.EVENT_RES_STATUS: event.status,
-        rpc_api.EVENT_RES_STATUS_DATA: event.reason,
+        rpc_api.EVENT_RES_ACTION: event.resource_action,
+        rpc_api.EVENT_RES_STATUS: event.resource_status,
+        rpc_api.EVENT_RES_STATUS_DATA: event.resource_status_reason,
         rpc_api.EVENT_RES_TYPE: event.resource_type,
-        rpc_api.EVENT_RES_PROPERTIES: event.resource_properties,
     }
+    if root_stack_identifier:
+        result[rpc_api.EVENT_ROOT_STACK_ID] = dict(root_stack_identifier)
+    if include_rsrc_prop_data:
+        result[rpc_api.EVENT_RES_PROPERTIES] = event.resource_properties
 
     return result
 
@@ -362,28 +408,40 @@ def format_notification_body(stack):
         state = '_'.join(stack.state)
     else:
         state = 'Unknown'
+
+    updated_at = heat_timeutils.isotime(stack.updated_time)
     result = {
         rpc_api.NOTIFY_TENANT_ID: stack.context.tenant_id,
-        rpc_api.NOTIFY_USER_ID: stack.context.user,
+        rpc_api.NOTIFY_USER_ID: stack.context.username,
+        # deprecated: please use rpc_api.NOTIFY_USERID for user id or
+        # rpc_api.NOTIFY_USERNAME for user name.
+        rpc_api.NOTIFY_USERID: stack.context.user_id,
+        rpc_api.NOTIFY_USERNAME: stack.context.username,
         rpc_api.NOTIFY_STACK_ID: stack.id,
         rpc_api.NOTIFY_STACK_NAME: stack.name,
         rpc_api.NOTIFY_STATE: state,
         rpc_api.NOTIFY_STATE_REASON: stack.status_reason,
-        rpc_api.NOTIFY_CREATE_AT: stack.created_time.isoformat(),
+        rpc_api.NOTIFY_CREATE_AT: heat_timeutils.isotime(stack.created_time),
+        rpc_api.NOTIFY_TAGS: stack.tags,
+        rpc_api.NOTIFY_UPDATE_AT: updated_at
     }
+    if stack.t is not None:
+        result[rpc_api.NOTIFY_DESCRIPTION] = stack.t[stack.t.DESCRIPTION]
+
     return result
 
 
 def format_watch(watch):
 
-    updated_at = watch.updated_at or timeutils.utcnow()
+    updated_time = heat_timeutils.isotime(watch.updated_at or
+                                          timeutils.utcnow())
     result = {
         rpc_api.WATCH_ACTIONS_ENABLED: watch.rule.get(
             rpc_api.RULE_ACTIONS_ENABLED),
         rpc_api.WATCH_ALARM_ACTIONS: watch.rule.get(
             rpc_api.RULE_ALARM_ACTIONS),
         rpc_api.WATCH_TOPIC: watch.rule.get(rpc_api.RULE_TOPIC),
-        rpc_api.WATCH_UPDATED_TIME: updated_at.isoformat(),
+        rpc_api.WATCH_UPDATED_TIME: updated_time,
         rpc_api.WATCH_DESCRIPTION: watch.rule.get(rpc_api.RULE_DESCRIPTION),
         rpc_api.WATCH_NAME: watch.name,
         rpc_api.WATCH_COMPARISON: watch.rule.get(rpc_api.RULE_COMPARISON),
@@ -399,8 +457,9 @@ def format_watch(watch):
         rpc_api.WATCH_STATE_REASON: watch.rule.get(rpc_api.RULE_STATE_REASON),
         rpc_api.WATCH_STATE_REASON_DATA:
         watch.rule.get(rpc_api.RULE_STATE_REASON_DATA),
-        rpc_api.WATCH_STATE_UPDATED_TIME: watch.rule.get(
-            rpc_api.RULE_STATE_UPDATED_TIME, timeutils.utcnow()).isoformat(),
+        rpc_api.WATCH_STATE_UPDATED_TIME: heat_timeutils.isotime(
+            watch.rule.get(rpc_api.RULE_STATE_UPDATED_TIME,
+                           timeutils.utcnow())),
         rpc_api.WATCH_STATE_VALUE: watch.state,
         rpc_api.WATCH_STATISTIC: watch.rule.get(rpc_api.RULE_STATISTIC),
         rpc_api.WATCH_THRESHOLD: watch.rule.get(rpc_api.RULE_THRESHOLD),
@@ -411,7 +470,7 @@ def format_watch(watch):
     return result
 
 
-def format_watch_data(wd):
+def format_watch_data(wd, rule_names):
 
     # Demangle DB format data into something more easily used in the API
     # We are expecting a dict with exactly two items, Namespace and
@@ -421,13 +480,13 @@ def format_watch_data(wd):
     if len(metric) == 1:
         metric_name, metric_data = metric[0]
     else:
-        LOG.error(_LE("Unexpected number of keys in watch_data.data!"))
+        LOG.error("Unexpected number of keys in watch_data.data!")
         return
 
     result = {
-        rpc_api.WATCH_DATA_ALARM: wd.watch_rule.name,
+        rpc_api.WATCH_DATA_ALARM: rule_names.get(wd.watch_rule_id),
         rpc_api.WATCH_DATA_METRIC: metric_name,
-        rpc_api.WATCH_DATA_TIME: wd.created_at.isoformat(),
+        rpc_api.WATCH_DATA_TIME: heat_timeutils.isotime(wd.created_at),
         rpc_api.WATCH_DATA_NAMESPACE: namespace,
         rpc_api.WATCH_DATA: metric_data
     }
@@ -467,6 +526,15 @@ def format_validate_parameter(param):
     if param.user_value:
         res[rpc_api.PARAM_VALUE] = param.user_value
 
+    if param.tags():
+        res[rpc_api.PARAM_TAG] = param.tags()
+
+    _build_parameter_constraints(res, param)
+
+    return res
+
+
+def _build_parameter_constraints(res, param):
     constraint_description = []
 
     # build constraints
@@ -485,6 +553,13 @@ def format_validate_parameter(param):
             if c.max is not None:
                 res[rpc_api.PARAM_MAX_VALUE] = c.max
 
+        elif isinstance(c, constr.Modulo):
+            if c.step is not None:
+                res[rpc_api.PARAM_STEP] = c.step
+
+            if c.offset is not None:
+                res[rpc_api.PARAM_OFFSET] = c.offset
+
         elif isinstance(c, constr.AllowedValues):
             res[rpc_api.PARAM_ALLOWED_VALUES] = list(c.allowed)
 
@@ -501,23 +576,25 @@ def format_validate_parameter(param):
         res[rpc_api.PARAM_CONSTRAINT_DESCRIPTION] = " ".join(
             constraint_description)
 
-    return res
 
-
-def format_software_config(sc, detail=True):
+def format_software_config(sc, detail=True, include_project=False):
     if sc is None:
         return
     result = {
         rpc_api.SOFTWARE_CONFIG_ID: sc.id,
         rpc_api.SOFTWARE_CONFIG_NAME: sc.name,
         rpc_api.SOFTWARE_CONFIG_GROUP: sc.group,
-        rpc_api.SOFTWARE_CONFIG_CREATION_TIME: sc.created_at.isoformat()
+        rpc_api.SOFTWARE_CONFIG_CREATION_TIME:
+            heat_timeutils.isotime(sc.created_at)
     }
     if detail:
         result[rpc_api.SOFTWARE_CONFIG_CONFIG] = sc.config['config']
         result[rpc_api.SOFTWARE_CONFIG_INPUTS] = sc.config['inputs']
         result[rpc_api.SOFTWARE_CONFIG_OUTPUTS] = sc.config['outputs']
         result[rpc_api.SOFTWARE_CONFIG_OPTIONS] = sc.config['options']
+
+    if include_project:
+        result[rpc_api.SOFTWARE_CONFIG_PROJECT] = sc.tenant
     return result
 
 
@@ -533,11 +610,12 @@ def format_software_deployment(sd):
         rpc_api.SOFTWARE_DEPLOYMENT_STATUS: sd.status,
         rpc_api.SOFTWARE_DEPLOYMENT_STATUS_REASON: sd.status_reason,
         rpc_api.SOFTWARE_DEPLOYMENT_CONFIG_ID: sd.config.id,
-        rpc_api.SOFTWARE_DEPLOYMENT_CREATION_TIME: sd.created_at.isoformat(),
+        rpc_api.SOFTWARE_DEPLOYMENT_CREATION_TIME:
+            heat_timeutils.isotime(sd.created_at),
     }
     if sd.updated_at:
         result[rpc_api.SOFTWARE_DEPLOYMENT_UPDATED_TIME] = (
-            sd.updated_at.isoformat())
+            heat_timeutils.isotime(sd.updated_at))
     return result
 
 
@@ -550,6 +628,7 @@ def format_snapshot(snapshot):
         rpc_api.SNAPSHOT_STATUS: snapshot.status,
         rpc_api.SNAPSHOT_STATUS_REASON: snapshot.status_reason,
         rpc_api.SNAPSHOT_DATA: snapshot.data,
-        rpc_api.SNAPSHOT_CREATION_TIME: snapshot.created_at.isoformat(),
+        rpc_api.SNAPSHOT_CREATION_TIME:
+            heat_timeutils.isotime(snapshot.created_at),
     }
     return result
