@@ -11,8 +11,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import six
-
 from heat.common import exception
 from heat.common.i18n import _
 from heat.engine import attributes
@@ -22,12 +20,41 @@ from heat.engine import properties
 from heat.engine import resource
 from heat.engine.resources.openstack.neutron import neutron
 from heat.engine import support
+from heat.engine import translation
+
+DEPR_MSG = _('Neutron LBaaS v1 is deprecated in the Liberty release '
+             'and is planned to be removed in a future release. '
+             'Going forward, the LBaaS V2 should be used.')
 
 
 class HealthMonitor(neutron.NeutronResource):
+    """A resource for managing health monitors for loadbalancers in Neutron.
+
+    A health monitor is used to determine whether or not back-end members of
+    the VIP's pool are usable for processing a request. A pool can have several
+    health monitors associated with it. There are different types of health
+    monitors supported by the OpenStack LBaaS service:
+
+      - PING: used to ping the members using ICMP.
+      - TCP: used to connect to the members using TCP.
+      - HTTP: used to send an HTTP request to the member.
+      - HTTPS: used to send a secure HTTP request to the member.
     """
-    A resource for managing health monitors for load balancers in Neutron.
-    """
+
+    required_service_extension = 'lbaas'
+
+    entity = 'health_monitor'
+
+    support_status = support.SupportStatus(
+        status=support.HIDDEN,
+        version='9.0.0',
+        message=_('Use LBaaS V2 instead.'),
+        previous_status=support.SupportStatus(
+            status=support.DEPRECATED,
+            message=DEPR_MSG,
+            version='7.0.0'
+        )
+    )
 
     PROPERTIES = (
         DELAY, TYPE, MAX_RETRIES, TIMEOUT, ADMIN_STATE_UP,
@@ -48,8 +75,8 @@ class HealthMonitor(neutron.NeutronResource):
     properties_schema = {
         DELAY: properties.Schema(
             properties.Schema.INTEGER,
-            _('The minimum time in seconds between regular connections of '
-              'the member.'),
+            _('The minimum time in milliseconds between regular connections '
+              'of the member.'),
             required=True,
             update_allowed=True
         ),
@@ -70,7 +97,7 @@ class HealthMonitor(neutron.NeutronResource):
         ),
         TIMEOUT: properties.Schema(
             properties.Schema.INTEGER,
-            _('Maximum number of seconds for a monitor to wait for a '
+            _('Maximum number of milliseconds for a monitor to wait for a '
               'connection to be established before it times out.'),
             required=True,
             update_allowed=True
@@ -107,7 +134,7 @@ class HealthMonitor(neutron.NeutronResource):
             type=attributes.Schema.STRING
         ),
         DELAY_ATTR: attributes.Schema(
-            _('The minimum time in seconds between regular connections '
+            _('The minimum time in milliseconds between regular connections '
               'of the member.'),
             type=attributes.Schema.STRING
         ),
@@ -127,7 +154,7 @@ class HealthMonitor(neutron.NeutronResource):
             type=attributes.Schema.STRING
         ),
         TIMEOUT_ATTR: attributes.Schema(
-            _('Maximum number of seconds for a monitor to wait for a '
+            _('Maximum number of milliseconds for a monitor to wait for a '
               'connection to be established before it times out.'),
             type=attributes.Schema.STRING
         ),
@@ -154,10 +181,6 @@ class HealthMonitor(neutron.NeutronResource):
             {'health_monitor': properties})['health_monitor']
         self.resource_id_set(health_monitor['id'])
 
-    def _show_resource(self):
-        return self.client().show_health_monitor(
-            self.resource_id)['health_monitor']
-
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
         if prop_diff:
             self.client().update_health_monitor(
@@ -173,9 +196,29 @@ class HealthMonitor(neutron.NeutronResource):
 
 
 class Pool(neutron.NeutronResource):
+    """A resource for managing load balancer pools in Neutron.
+
+    A load balancing pool is a logical set of devices, such as web servers,
+    that you group together to receive and process traffic. The loadbalancing
+    function chooses a member of the pool according to the configured load
+    balancing method to handle the new requests or connections received on the
+    VIP address. There is only one pool for a VIP.
     """
-    A resource for managing load balancer pools in Neutron.
-    """
+
+    required_service_extension = 'lbaas'
+
+    entity = 'pool'
+
+    support_status = support.SupportStatus(
+        status=support.HIDDEN,
+        version='9.0.0',
+        message=_('Use LBaaS V2 instead.'),
+        previous_status=support.SupportStatus(
+            status=support.DEPRECATED,
+            message=DEPR_MSG,
+            version='7.0.0'
+        )
+    )
 
     PROPERTIES = (
         PROTOCOL, SUBNET_ID, SUBNET, LB_METHOD, NAME, DESCRIPTION,
@@ -238,6 +281,7 @@ class Pool(neutron.NeutronResource):
             _('The subnet for the port on which the members '
               'of the pool will be connected.'),
             support_status=support.SupportStatus(version='2014.2'),
+            required=True,
             constraints=[
                 constraints.CustomConstraint('neutron.subnet')
             ]
@@ -272,6 +316,9 @@ class Pool(neutron.NeutronResource):
             properties.Schema.STRING,
             _('LBaaS provider to implement this load balancer instance.'),
             support_status=support.SupportStatus(version='5.0.0'),
+            constraints=[
+                constraints.CustomConstraint('neutron.lb.provider')
+            ],
         ),
         VIP: properties.Schema(
             properties.Schema.MAP,
@@ -388,13 +435,30 @@ class Pool(neutron.NeutronResource):
         ),
     }
 
-    def translation_rules(self):
+    def translation_rules(self, props):
+        client_plugin = self.client_plugin()
         return [
-            properties.TranslationRule(
-                self.properties,
-                properties.TranslationRule.REPLACE,
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.REPLACE,
                 [self.SUBNET],
                 value_path=[self.SUBNET_ID]
+            ),
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.RESOLVE,
+                [self.SUBNET],
+                client_plugin=client_plugin,
+                finder='find_resourceid_by_name_or_id',
+                entity=client_plugin.RES_TYPE_SUBNET
+            ),
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.RESOLVE,
+                [self.VIP, self.VIP_SUBNET],
+                client_plugin=self.client_plugin(),
+                finder='find_resourceid_by_name_or_id',
+                entity=client_plugin.RES_TYPE_SUBNET
             )
         ]
 
@@ -402,8 +466,6 @@ class Pool(neutron.NeutronResource):
         res = super(Pool, self).validate()
         if res:
             return res
-        self._validate_depr_property_required(
-            self.properties, self.SUBNET, self.SUBNET_ID)
         session_p = self.properties[self.VIP].get(self.VIP_SESSION_PERSISTENCE)
         if session_p is None:
             # session persistence is not configured, skip validation
@@ -422,8 +484,8 @@ class Pool(neutron.NeutronResource):
         properties = self.prepare_properties(
             self.properties,
             self.physical_resource_name())
-        self.client_plugin().resolve_subnet(
-            properties, self.SUBNET, 'subnet_id')
+        subnet_id = properties.pop(self.SUBNET)
+        properties['subnet_id'] = subnet_id
         vip_properties = properties.pop(self.VIP)
         monitors = properties.pop(self.MONITORS)
 
@@ -446,19 +508,15 @@ class Pool(neutron.NeutronResource):
         vip_arguments['protocol'] = self.properties[self.PROTOCOL]
 
         if vip_arguments.get(self.VIP_SUBNET) is None:
-            vip_arguments['subnet_id'] = properties[self.SUBNET_ID]
+            vip_arguments['subnet_id'] = subnet_id
         else:
-            vip_arguments['subnet_id'] = self.client_plugin().resolve_subnet(
-                vip_arguments, self.VIP_SUBNET, 'subnet_id')
+            vip_arguments['subnet_id'] = vip_arguments.pop(self.VIP_SUBNET)
 
         vip_arguments['pool_id'] = pool['id']
         vip = self.client().create_vip({'vip': vip_arguments})['vip']
 
         self.metadata_set({'vip': vip['id']})
 
-    def _show_resource(self):
-        return self.client().show_pool(self.resource_id)['pool']
-
     def check_create_complete(self, data):
         attributes = self._show_resource()
         status = attributes['status']
@@ -551,11 +609,26 @@ class Pool(neutron.NeutronResource):
 
 
 class PoolMember(neutron.NeutronResource):
-    """
-    A resource to handle load balancer members.
+    """A resource to handle loadbalancer members.
+
+    A pool member represents the application running on backend server.
     """
 
-    support_status = support.SupportStatus(version='2014.1')
+    required_service_extension = 'lbaas'
+
+    entity = 'member'
+
+    support_status = support.SupportStatus(
+        status=support.HIDDEN,
+        version='9.0.0',
+        message=_('Use LBaaS V2 instead.'),
+        previous_status=support.SupportStatus(
+            status=support.DEPRECATED,
+            message=DEPR_MSG,
+            version='7.0.0',
+            previous_status=support.SupportStatus(version='2014.1')
+        )
+    )
 
     PROPERTIES = (
         POOL_ID, ADDRESS, PROTOCOL_PORT, WEIGHT, ADMIN_STATE_UP,
@@ -658,9 +731,6 @@ class PoolMember(neutron.NeutronResource):
         member = self.client().create_member({'member': params})['member']
         self.resource_id_set(member['id'])
 
-    def _show_resource(self):
-        return self.client().show_member(self.resource_id)['member']
-
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
         if prop_diff:
             self.client().update_member(
@@ -676,9 +746,25 @@ class PoolMember(neutron.NeutronResource):
 
 
 class LoadBalancer(resource.Resource):
+    """A resource to link a neutron pool with servers.
+
+    A loadbalancer allows linking a neutron pool with specified servers to some
+    port.
     """
-    A resource to link a neutron pool with servers.
-    """
+
+    required_service_extension = 'lbaas'
+
+    support_status = support.SupportStatus(
+        status=support.HIDDEN,
+        version='9.0.0',
+        message=_('Use LBaaS V2 instead.'),
+        previous_status=support.SupportStatus(
+            status=support.DEPRECATED,
+            message=DEPR_MSG,
+            version='7.0.0',
+            previous_status=support.SupportStatus(version='2014.1')
+        )
+    )
 
     PROPERTIES = (
         POOL_ID, PROTOCOL_PORT, MEMBERS,
@@ -737,13 +823,11 @@ class LoadBalancer(resource.Resource):
                  new_props[self.MEMBERS] is not None)):
             members = set(new_props[self.MEMBERS] or [])
             rd_members = self.data()
-            old_members = set(six.iterkeys(rd_members))
+            old_members = set(rd_members)
             for member in old_members - members:
                 member_id = rd_members[member]
-                try:
+                with self.client_plugin().ignore_not_found:
                     self.client().delete_member(member_id)
-                except Exception as ex:
-                    self.client_plugin().ignore_not_found(ex)
                 self.data_delete(member)
             pool = self.properties[self.POOL_ID]
             protocol_port = self.properties[self.PROTOCOL_PORT]
@@ -761,10 +845,8 @@ class LoadBalancer(resource.Resource):
         # FIXME(pshchelo): this deletes members in a tight loop,
         # so is prone to OverLimit bug similar to LP 1265937
         for member, member_id in self.data().items():
-            try:
+            with self.client_plugin().ignore_not_found:
                 self.client().delete_member(member_id)
-            except Exception as ex:
-                self.client_plugin().ignore_not_found(ex)
             self.data_delete(member)
 
 
