@@ -11,18 +11,21 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
+
 from oslo_config import cfg
 
 from heat.common import exception
 from heat.common import short_id
 from heat.common import template_format
+from heat.engine.clients.os.keystone import fake_keystoneclient as fake_ks
+from heat.engine import node_data
 from heat.engine.resources.aws.iam import user
 from heat.engine.resources.openstack.heat import access_policy as ap
 from heat.engine import scheduler
+from heat.engine import stk_defn
 from heat.objects import resource_data as resource_data_object
 from heat.tests import common
-from heat.tests import fakes
 from heat.tests import utils
 
 
@@ -107,35 +110,30 @@ class UserTest(common.HeatTestCase):
         super(UserTest, self).setUp()
         self.stack_name = 'test_user_stack_%s' % utils.random_name()
         self.username = '%s-CfnUser-aabbcc' % self.stack_name
-        self.fc = fakes.FakeKeystoneClient(username=self.username)
+        self.fc = fake_ks.FakeKeystoneClient(username=self.username)
         cfg.CONF.set_default('heat_stack_user_role', 'stack_user_role')
 
     def create_user(self, t, stack, resource_name,
-                    project_id='stackproject', user_id='dummy_user',
+                    project_id, user_id='dummy_user',
                     password=None):
-        self.m.StubOutWithMock(user.User, 'keystone')
-        user.User.keystone().MultipleTimes().AndReturn(self.fc)
+        self.patchobject(user.User, 'keystone', return_value=self.fc)
 
-        self.m.StubOutWithMock(fakes.FakeKeystoneClient,
-                               'create_stack_domain_project')
-        fakes.FakeKeystoneClient.create_stack_domain_project(
-            stack.id).AndReturn(project_id)
+        self.mock_create_project = self.patchobject(
+            fake_ks.FakeKeystoneClient, 'create_stack_domain_project',
+            return_value=project_id)
 
         resource_defns = stack.t.resource_definitions(stack)
         rsrc = user.User(resource_name,
                          resource_defns[resource_name],
                          stack)
-        rsrc._store()
+        rsrc.store()
 
-        self.m.StubOutWithMock(short_id, 'get_id')
-        short_id.get_id(rsrc.uuid).MultipleTimes().AndReturn('aabbcc')
+        self.patchobject(short_id, 'get_id', return_value='aabbcc')
+
+        self.mock_create_user = self.patchobject(
+            fake_ks.FakeKeystoneClient, 'create_stack_domain_user',
+            return_value=user_id)
 
-        self.m.StubOutWithMock(fakes.FakeKeystoneClient,
-                               'create_stack_domain_user')
-        fakes.FakeKeystoneClient.create_stack_domain_user(
-            username=self.username, password=password,
-            project_id=project_id).AndReturn(user_id)
-        self.m.ReplayAll()
         self.assertIsNone(rsrc.validate())
         scheduler.TaskRunner(rsrc.create)()
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
@@ -144,8 +142,9 @@ class UserTest(common.HeatTestCase):
     def test_user(self):
         t = template_format.parse(user_template)
         stack = utils.parse_stack(t, stack_name=self.stack_name)
+        project_id = 'stackproject'
 
-        rsrc = self.create_user(t, stack, 'CfnUser')
+        rsrc = self.create_user(t, stack, 'CfnUser', project_id)
         self.assertEqual('dummy_user', rsrc.resource_id)
         self.assertEqual(self.username, rsrc.FnGetRefId())
 
@@ -173,24 +172,34 @@ class UserTest(common.HeatTestCase):
 
         scheduler.TaskRunner(rsrc.delete)()
         self.assertEqual((rsrc.DELETE, rsrc.COMPLETE), rsrc.state)
-        self.m.VerifyAll()
+        self.mock_create_project.assert_called_once_with(stack.id)
+        self.mock_create_user.assert_called_once_with(
+            password=None, project_id=project_id,
+            username=self.username)
 
     def test_user_password(self):
         t = template_format.parse(user_template_password)
         stack = utils.parse_stack(t, stack_name=self.stack_name)
-
-        rsrc = self.create_user(t, stack, 'CfnUser', password=u'myP@ssW0rd')
+        project_id = 'stackproject'
+        password = u'myP@ssW0rd'
+        rsrc = self.create_user(t, stack, 'CfnUser',
+                                project_id=project_id,
+                                password=password)
         self.assertEqual('dummy_user', rsrc.resource_id)
         self.assertEqual(self.username, rsrc.FnGetRefId())
 
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
-        self.m.VerifyAll()
+        self.mock_create_project.assert_called_once_with(stack.id)
+        self.mock_create_user.assert_called_once_with(
+            password=password, project_id=project_id,
+            username=self.username)
 
     def test_user_validate_policies(self):
         t = template_format.parse(user_policy_template)
         stack = utils.parse_stack(t, stack_name=self.stack_name)
+        project_id = 'stackproject'
 
-        rsrc = self.create_user(t, stack, 'CfnUser')
+        rsrc = self.create_user(t, stack, 'CfnUser', project_id)
         self.assertEqual('dummy_user', rsrc.resource_id)
         self.assertEqual(self.username, rsrc.FnGetRefId())
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
@@ -218,8 +227,10 @@ class UserTest(common.HeatTestCase):
 
         # However we should just ignore it to avoid breaking existing templates
         self.assertTrue(rsrc._validate_policies([dict_policy]))
-
-        self.m.VerifyAll()
+        self.mock_create_project.assert_called_once_with(stack.id)
+        self.mock_create_user.assert_called_once_with(
+            password=None, project_id=project_id,
+            username=self.username)
 
     def test_user_create_bad_policies(self):
         t = template_format.parse(user_policy_template)
@@ -234,44 +245,54 @@ class UserTest(common.HeatTestCase):
                           rsrc.handle_create)
 
     def test_user_access_allowed(self):
-        self.m.StubOutWithMock(ap.AccessPolicy, 'access_allowed')
-        ap.AccessPolicy.access_allowed('a_resource').AndReturn(True)
-        ap.AccessPolicy.access_allowed('b_resource').AndReturn(False)
 
-        self.m.ReplayAll()
+        def mock_access_allowed(resource):
+            return True if resource == 'a_resource' else False
+
+        self.patchobject(ap.AccessPolicy, 'access_allowed',
+                         side_effect=mock_access_allowed)
 
         t = template_format.parse(user_policy_template)
         stack = utils.parse_stack(t, stack_name=self.stack_name)
+        project_id = 'stackproject'
 
-        rsrc = self.create_user(t, stack, 'CfnUser')
+        rsrc = self.create_user(t, stack, 'CfnUser', project_id)
         self.assertEqual('dummy_user', rsrc.resource_id)
         self.assertEqual(self.username, rsrc.FnGetRefId())
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
 
         self.assertTrue(rsrc.access_allowed('a_resource'))
         self.assertFalse(rsrc.access_allowed('b_resource'))
-        self.m.VerifyAll()
+        self.mock_create_project.assert_called_once_with(stack.id)
+        self.mock_create_user.assert_called_once_with(
+            password=None, project_id=project_id,
+            username=self.username)
 
     def test_user_access_allowed_ignorepolicy(self):
-        self.m.StubOutWithMock(ap.AccessPolicy, 'access_allowed')
-        ap.AccessPolicy.access_allowed('a_resource').AndReturn(True)
-        ap.AccessPolicy.access_allowed('b_resource').AndReturn(False)
 
-        self.m.ReplayAll()
+        def mock_access_allowed(resource):
+            return True if resource == 'a_resource' else False
+
+        self.patchobject(ap.AccessPolicy, 'access_allowed',
+                         side_effect=mock_access_allowed)
 
         t = template_format.parse(user_policy_template)
         t['Resources']['CfnUser']['Properties']['Policies'] = [
             'WebServerAccessPolicy', {'an_ignored': 'policy'}]
         stack = utils.parse_stack(t, stack_name=self.stack_name)
+        project_id = 'stackproject'
 
-        rsrc = self.create_user(t, stack, 'CfnUser')
+        rsrc = self.create_user(t, stack, 'CfnUser', project_id)
         self.assertEqual('dummy_user', rsrc.resource_id)
         self.assertEqual(self.username, rsrc.FnGetRefId())
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
 
         self.assertTrue(rsrc.access_allowed('a_resource'))
         self.assertFalse(rsrc.access_allowed('b_resource'))
-        self.m.VerifyAll()
+        self.mock_create_project.assert_called_once_with(stack.id)
+        self.mock_create_user.assert_called_once_with(
+            password=None, project_id=project_id,
+            username=self.username)
 
     def test_user_refid_rsrc_id(self):
         t = template_format.parse(user_template)
@@ -282,15 +303,15 @@ class UserTest(common.HeatTestCase):
 
     def test_user_refid_convg_cache_data(self):
         t = template_format.parse(user_template)
-        cache_data = {'CfnUser': {
+        cache_data = {'CfnUser': node_data.NodeData.from_dict({
             'uuid': mock.ANY,
             'id': mock.ANY,
             'action': 'CREATE',
             'status': 'COMPLETE',
             'reference_id': 'convg_xyz'
-        }}
+        })}
         stack = utils.parse_stack(t, cache_data=cache_data)
-        rsrc = stack['CfnUser']
+        rsrc = stack.defn['CfnUser']
         self.assertEqual('convg_xyz', rsrc.FnGetRefId())
 
 
@@ -299,29 +320,25 @@ class AccessKeyTest(common.HeatTestCase):
         super(AccessKeyTest, self).setUp()
         self.username = utils.PhysName('test_stack', 'CfnUser')
         self.credential_id = 'acredential123'
-        self.fc = fakes.FakeKeystoneClient(username=self.username,
-                                           user_id='dummy_user',
-                                           credential_id=self.credential_id)
+        self.fc = fake_ks.FakeKeystoneClient(username=self.username,
+                                             user_id='dummy_user',
+                                             credential_id=self.credential_id)
         cfg.CONF.set_default('heat_stack_user_role', 'stack_user_role')
 
     def create_user(self, t, stack, resource_name,
                     project_id='stackproject', user_id='dummy_user',
                     password=None):
-        self.m.StubOutWithMock(user.User, 'keystone')
-        user.User.keystone().MultipleTimes().AndReturn(self.fc)
-
-        self.m.ReplayAll()
+        self.patchobject(user.User, 'keystone', return_value=self.fc)
         rsrc = stack[resource_name]
         self.assertIsNone(rsrc.validate())
         scheduler.TaskRunner(rsrc.create)()
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
+        stk_defn.update_resource_data(stack.defn, resource_name,
+                                      rsrc.node_data())
         return rsrc
 
     def create_access_key(self, t, stack, resource_name):
-        resource_defns = stack.t.resource_definitions(stack)
-        rsrc = user.AccessKey(resource_name,
-                              resource_defns[resource_name],
-                              stack)
+        rsrc = stack[resource_name]
         self.assertIsNone(rsrc.validate())
         scheduler.TaskRunner(rsrc.create)()
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
@@ -333,8 +350,6 @@ class AccessKeyTest(common.HeatTestCase):
 
         self.create_user(t, stack, 'CfnUser')
         rsrc = self.create_access_key(t, stack, 'HostKeys')
-
-        self.m.VerifyAll()
         self.assertEqual(self.fc.access,
                          rsrc.resource_id)
 
@@ -358,14 +373,9 @@ class AccessKeyTest(common.HeatTestCase):
 
         scheduler.TaskRunner(rsrc.delete)()
         self.assertEqual((rsrc.DELETE, rsrc.COMPLETE), rsrc.state)
-        self.m.VerifyAll()
 
     def test_access_key_get_from_keystone(self):
-        self.m.StubOutWithMock(user.AccessKey, 'keystone')
-        user.AccessKey.keystone().MultipleTimes().AndReturn(self.fc)
-
-        self.m.ReplayAll()
-
+        self.patchobject(user.AccessKey, 'keystone', return_value=self.fc)
         t = template_format.parse(user_accesskey_template)
 
         stack = utils.parse_stack(t)
@@ -389,11 +399,8 @@ class AccessKeyTest(common.HeatTestCase):
 
         scheduler.TaskRunner(rsrc.delete)()
         self.assertEqual((rsrc.DELETE, rsrc.COMPLETE), rsrc.state)
-        self.m.VerifyAll()
 
     def test_access_key_no_user(self):
-        self.m.ReplayAll()
-
         t = template_format.parse(user_accesskey_template)
         # Set the resource properties UserName to an unknown user
         t['Resources']['HostKeys']['Properties']['UserName'] = 'NonExistent'
@@ -411,8 +418,6 @@ class AccessKeyTest(common.HeatTestCase):
         scheduler.TaskRunner(rsrc.delete)()
         self.assertEqual((rsrc.DELETE, rsrc.COMPLETE), rsrc.state)
 
-        self.m.VerifyAll()
-
 
 class AccessPolicyTest(common.HeatTestCase):
 
