@@ -11,18 +11,21 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from unittest import mock
+
 from oslo_serialization import jsonutils
-import six
 
 from heat.common import exception
-from heat.engine.cfn import functions as cfn_funcs
 from heat.engine import constraints
+from heat.engine.hot import functions as hot_funcs
 from heat.engine.hot import parameters as hot_param
 from heat.engine import parameters
 from heat.engine import plugin_manager
 from heat.engine import properties
 from heat.engine import resources
+from heat.engine import rsrc_defn
 from heat.engine import support
+from heat.engine import translation
 from heat.tests import common
 
 
@@ -68,8 +71,8 @@ class PropertySchemaTest(common.HeatTestCase):
         s = properties.Schema(properties.Schema.STRING, 'A string',
                               default='wibble',
                               constraints=[constraints.Length(4, 8)])
-        l = properties.Schema(properties.Schema.LIST, 'A list', schema=s)
-        self.assertEqual(d, dict(l))
+        ls = properties.Schema(properties.Schema.LIST, 'A list', schema=s)
+        self.assertEqual(d, dict(ls))
 
     def test_schema_map_schema(self):
         d = {
@@ -134,14 +137,14 @@ class PropertySchemaTest(common.HeatTestCase):
                               constraints=[constraints.Length(4, 8)])
         m = properties.Schema(properties.Schema.MAP, 'A map',
                               schema={'Foo': s})
-        l = properties.Schema(properties.Schema.LIST, 'A list', schema=m)
-        self.assertEqual(d, dict(l))
+        ls = properties.Schema(properties.Schema.LIST, 'A list', schema=m)
+        self.assertEqual(d, dict(ls))
 
     def test_all_resource_schemata(self):
         for resource_type in resources.global_env().get_types():
-            for schema in six.itervalues(getattr(resource_type,
-                                                 'properties_schema',
-                                                 {})):
+            for schema in getattr(resource_type,
+                                  'properties_schema',
+                                  {}).values():
                 properties.Schema.from_legacy(schema)
 
     def test_from_legacy_idempotency(self):
@@ -289,7 +292,7 @@ class PropertySchemaTest(common.HeatTestCase):
         self.assertEqual('[a-z]*', c.pattern)
 
     def test_from_legacy_list(self):
-        l = properties.Schema.from_legacy({
+        ls = properties.Schema.from_legacy({
             'Type': 'List',
             'Default': ['wibble'],
             'Schema': {
@@ -298,15 +301,15 @@ class PropertySchemaTest(common.HeatTestCase):
                 'MaxLength': 8,
             }
         })
-        self.assertEqual(properties.Schema.LIST, l.type)
-        self.assertEqual(['wibble'], l.default)
+        self.assertEqual(properties.Schema.LIST, ls.type)
+        self.assertEqual(['wibble'], ls.default)
 
-        ss = l.schema[0]
+        ss = ls.schema[0]
         self.assertEqual(properties.Schema.STRING, ss.type)
         self.assertEqual('wibble', ss.default)
 
     def test_from_legacy_map(self):
-        l = properties.Schema.from_legacy({
+        ls = properties.Schema.from_legacy({
             'Type': 'Map',
             'Schema': {
                 'foo': {
@@ -315,9 +318,9 @@ class PropertySchemaTest(common.HeatTestCase):
                 }
             }
         })
-        self.assertEqual(properties.Schema.MAP, l.type)
+        self.assertEqual(properties.Schema.MAP, ls.type)
 
-        ss = l.schema['foo']
+        ss = ls.schema['foo']
         self.assertEqual(properties.Schema.STRING, ss.type)
         self.assertEqual('wibble', ss.default)
 
@@ -344,7 +347,7 @@ class PropertySchemaTest(common.HeatTestCase):
 
         self.assertEqual(properties.Schema.STRING, schema.type)
         self.assertEqual(description, schema.description)
-        self.assertEqual("m1.large", schema.default)
+        self.assertIsNone(schema.default)
         self.assertFalse(schema.required)
         self.assertEqual(1, len(schema.constraints))
 
@@ -353,6 +356,9 @@ class PropertySchemaTest(common.HeatTestCase):
         self.assertEqual(tuple(allowed_values), allowed_constraint.allowed)
         self.assertEqual(constraint_desc, allowed_constraint.description)
 
+        props = properties.Properties({'test': schema}, {})
+        props.validate()
+
     def test_from_string_allowed_pattern(self):
         description = "WebServer EC2 instance type"
         allowed_pattern = "[A-Za-z0-9.]*"
@@ -369,7 +375,7 @@ class PropertySchemaTest(common.HeatTestCase):
 
         self.assertEqual(properties.Schema.STRING, schema.type)
         self.assertEqual(description, schema.description)
-        self.assertEqual("m1.large", schema.default)
+        self.assertIsNone(schema.default)
         self.assertFalse(schema.required)
         self.assertEqual(1, len(schema.constraints))
 
@@ -378,6 +384,9 @@ class PropertySchemaTest(common.HeatTestCase):
         self.assertEqual(allowed_pattern, allowed_constraint.pattern)
         self.assertEqual(constraint_desc, allowed_constraint.description)
 
+        props = properties.Properties({'test': schema}, {})
+        props.validate()
+
     def test_from_string_multi_constraints(self):
         description = "WebServer EC2 instance type"
         allowed_pattern = "[A-Za-z0-9.]*"
@@ -395,7 +404,7 @@ class PropertySchemaTest(common.HeatTestCase):
 
         self.assertEqual(properties.Schema.STRING, schema.type)
         self.assertEqual(description, schema.description)
-        self.assertEqual("m1.large", schema.default)
+        self.assertIsNone(schema.default)
         self.assertFalse(schema.required)
         self.assertEqual(2, len(schema.constraints))
 
@@ -407,6 +416,9 @@ class PropertySchemaTest(common.HeatTestCase):
         self.assertEqual(allowed_pattern, allowed_constraint.pattern)
         self.assertEqual(constraint_desc, allowed_constraint.description)
 
+        props = properties.Properties({'test': schema}, {})
+        props.validate()
+
     def test_from_param_string_min_len(self):
         param = parameters.Schema.from_dict('name', {
             "Description": "WebServer EC2 instance type",
@@ -417,6 +429,7 @@ class PropertySchemaTest(common.HeatTestCase):
         schema = properties.Schema.from_parameter(param)
 
         self.assertFalse(schema.required)
+        self.assertIsNone(schema.default)
         self.assertEqual(1, len(schema.constraints))
 
         len_constraint = schema.constraints[0]
@@ -424,6 +437,9 @@ class PropertySchemaTest(common.HeatTestCase):
         self.assertEqual(7, len_constraint.min)
         self.assertIsNone(len_constraint.max)
 
+        props = properties.Properties({'test': schema}, {})
+        props.validate()
+
     def test_from_param_string_max_len(self):
         param = parameters.Schema.from_dict('name', {
             "Description": "WebServer EC2 instance type",
@@ -434,6 +450,7 @@ class PropertySchemaTest(common.HeatTestCase):
         schema = properties.Schema.from_parameter(param)
 
         self.assertFalse(schema.required)
+        self.assertIsNone(schema.default)
         self.assertEqual(1, len(schema.constraints))
 
         len_constraint = schema.constraints[0]
@@ -441,6 +458,9 @@ class PropertySchemaTest(common.HeatTestCase):
         self.assertIsNone(len_constraint.min)
         self.assertEqual(11, len_constraint.max)
 
+        props = properties.Properties({'test': schema}, {})
+        props.validate()
+
     def test_from_param_string_min_max_len(self):
         param = parameters.Schema.from_dict('name', {
             "Description": "WebServer EC2 instance type",
@@ -452,6 +472,7 @@ class PropertySchemaTest(common.HeatTestCase):
         schema = properties.Schema.from_parameter(param)
 
         self.assertFalse(schema.required)
+        self.assertIsNone(schema.default)
         self.assertEqual(1, len(schema.constraints))
 
         len_constraint = schema.constraints[0]
@@ -459,6 +480,9 @@ class PropertySchemaTest(common.HeatTestCase):
         self.assertEqual(7, len_constraint.min)
         self.assertEqual(11, len_constraint.max)
 
+        props = properties.Properties({'test': schema}, {})
+        props.validate()
+
     def test_from_param_no_default(self):
         param = parameters.Schema.from_dict('name', {
             "Description": "WebServer EC2 instance type",
@@ -471,18 +495,20 @@ class PropertySchemaTest(common.HeatTestCase):
         self.assertEqual(0, len(schema.constraints))
         self.assertFalse(schema.allow_conversion)
 
+        props = properties.Properties({'name': schema}, {'name': 'm1.large'})
+        props.validate()
+
     def test_from_number_param_min(self):
-        default = "42"
         param = parameters.Schema.from_dict('name', {
             "Type": "Number",
-            "Default": default,
+            "Default": "42",
             "MinValue": "10",
         })
 
         schema = properties.Schema.from_parameter(param)
 
         self.assertEqual(properties.Schema.NUMBER, schema.type)
-        self.assertEqual(default, schema.default)
+        self.assertIsNone(schema.default)
         self.assertFalse(schema.required)
         self.assertEqual(1, len(schema.constraints))
 
@@ -491,18 +517,20 @@ class PropertySchemaTest(common.HeatTestCase):
         self.assertEqual(10, value_constraint.min)
         self.assertIsNone(value_constraint.max)
 
+        props = properties.Properties({'test': schema}, {})
+        props.validate()
+
     def test_from_number_param_max(self):
-        default = "42"
         param = parameters.Schema.from_dict('name', {
             "Type": "Number",
-            "Default": default,
+            "Default": "42",
             "MaxValue": "100",
         })
 
         schema = properties.Schema.from_parameter(param)
 
         self.assertEqual(properties.Schema.NUMBER, schema.type)
-        self.assertEqual(default, schema.default)
+        self.assertIsNone(schema.default)
         self.assertFalse(schema.required)
         self.assertEqual(1, len(schema.constraints))
 
@@ -511,11 +539,13 @@ class PropertySchemaTest(common.HeatTestCase):
         self.assertIsNone(value_constraint.min)
         self.assertEqual(100, value_constraint.max)
 
+        props = properties.Properties({'test': schema}, {})
+        props.validate()
+
     def test_from_number_param_min_max(self):
-        default = "42"
         param = parameters.Schema.from_dict('name', {
             "Type": "Number",
-            "Default": default,
+            "Default": "42",
             "MinValue": "10",
             "MaxValue": "100",
         })
@@ -523,7 +553,7 @@ class PropertySchemaTest(common.HeatTestCase):
         schema = properties.Schema.from_parameter(param)
 
         self.assertEqual(properties.Schema.NUMBER, schema.type)
-        self.assertEqual(default, schema.default)
+        self.assertIsNone(schema.default)
         self.assertFalse(schema.required)
         self.assertEqual(1, len(schema.constraints))
 
@@ -532,12 +562,14 @@ class PropertySchemaTest(common.HeatTestCase):
         self.assertEqual(10, value_constraint.min)
         self.assertEqual(100, value_constraint.max)
 
+        props = properties.Properties({'test': schema}, {})
+        props.validate()
+
     def test_from_number_param_allowed_vals(self):
-        default = "42"
         constraint_desc = "The quick brown fox jumps over the lazy dog."
         param = parameters.Schema.from_dict('name', {
             "Type": "Number",
-            "Default": default,
+            "Default": "42",
             "AllowedValues": ["10", "42", "100"],
             "ConstraintDescription": constraint_desc,
         })
@@ -545,7 +577,7 @@ class PropertySchemaTest(common.HeatTestCase):
         schema = properties.Schema.from_parameter(param)
 
         self.assertEqual(properties.Schema.NUMBER, schema.type)
-        self.assertEqual(default, schema.default)
+        self.assertIsNone(schema.default)
         self.assertFalse(schema.required)
         self.assertEqual(1, len(schema.constraints))
         self.assertFalse(schema.allow_conversion)
@@ -555,6 +587,9 @@ class PropertySchemaTest(common.HeatTestCase):
         self.assertEqual(('10', '42', '100'), allowed_constraint.allowed)
         self.assertEqual(constraint_desc, allowed_constraint.description)
 
+        props = properties.Properties({'test': schema}, {})
+        props.validate()
+
     def test_from_list_param(self):
         param = parameters.Schema.from_dict('name', {
             "Type": "CommaDelimitedList",
@@ -564,9 +599,12 @@ class PropertySchemaTest(common.HeatTestCase):
         schema = properties.Schema.from_parameter(param)
 
         self.assertEqual(properties.Schema.LIST, schema.type)
-        self.assertEqual("foo,bar,baz", schema.default)
+        self.assertIsNone(schema.default)
         self.assertFalse(schema.required)
-        self.assertFalse(schema.allow_conversion)
+        self.assertTrue(schema.allow_conversion)
+
+        props = properties.Properties({'test': schema}, {})
+        props.validate()
 
     def test_from_json_param(self):
         param = parameters.Schema.from_dict('name', {
@@ -577,11 +615,13 @@ class PropertySchemaTest(common.HeatTestCase):
         schema = properties.Schema.from_parameter(param)
 
         self.assertEqual(properties.Schema.MAP, schema.type)
-        self.assertEqual({"foo": "bar", "blarg": "wibble"},
-                         schema.default)
+        self.assertIsNone(schema.default)
         self.assertFalse(schema.required)
         self.assertTrue(schema.allow_conversion)
 
+        props = properties.Properties({'test': schema}, {})
+        props.validate()
+
     def test_no_mismatch_in_update_policy(self):
         manager = plugin_manager.PluginManager('heat.engine.resources')
         resource_mapping = plugin_manager.PluginMapping('resource')
@@ -596,7 +636,7 @@ class PropertySchemaTest(common.HeatTestCase):
                 update = True
             sub_schema = prop.schema
             if sub_schema:
-                for sub_prop_key, sub_prop in six.iteritems(sub_schema):
+                for sub_prop_key, sub_prop in sub_schema.items():
                     if not update:
                         self.assertEqual(update, sub_prop.update_allowed,
                                          "Mismatch in update policies: "
@@ -605,18 +645,18 @@ class PropertySchemaTest(common.HeatTestCase):
                                          {'res': resource_type,
                                           'prop': prop_key,
                                           'nested_prop': sub_prop_key})
-                    if sub_prop_key is '*':
+                    if sub_prop_key == '*':
                         check_update_policy(resource_type, prop_key,
                                             sub_prop, update)
                     else:
                         check_update_policy(resource_type, sub_prop_key,
                                             sub_prop, update)
 
-        for resource_type, resource_class in six.iteritems(all_resources):
+        for resource_type, resource_class in all_resources.items():
             props_schemata = properties.schemata(
                 resource_class.properties_schema)
 
-            for prop_key, prop in six.iteritems(props_schemata):
+            for prop_key, prop in props_schemata.items():
                 check_update_policy(resource_type, prop_key, prop)
 
 
@@ -744,9 +784,10 @@ class PropertyTest(common.HeatTestCase):
         p = properties.Property(schema)
         # python 3.4.3 returns another error message
         # try to handle this by regexp
-        self.assertRaisesRegexp(
-            TypeError, "int\(\) argument must be a string(, a bytes-like "
-                       "object)? or a number, not 'list'", p.get_value, [1])
+        self.assertRaisesRegex(
+            TypeError, r"int\(\) argument must be a string"
+                       "(, a bytes-like object)?"
+                       " or a number, not 'list'", p.get_value, [1])
 
     def test_str_from_int(self):
         schema = {'Type': 'String'}
@@ -767,7 +808,7 @@ class PropertyTest(common.HeatTestCase):
         schema = {'Type': 'Integer'}
         p = properties.Property(schema)
         ex = self.assertRaises(TypeError, p.get_value, '3a')
-        self.assertEqual("Value '3a' is not an integer", six.text_type(ex))
+        self.assertEqual("Value '3a' is not an integer", str(ex))
 
     def test_integer_low(self):
         schema = {'Type': 'Integer',
@@ -850,10 +891,14 @@ class PropertyTest(common.HeatTestCase):
         self.assertIs(False, p.get_value('false'))
         self.assertIs(False, p.get_value(False))
 
-    def test_boolean_invalid(self):
+    def test_boolean_invalid_string(self):
         p = properties.Property({'Type': 'Boolean'})
         self.assertRaises(ValueError, p.get_value, 'fish')
 
+    def test_boolean_invalid_int(self):
+        p = properties.Property({'Type': 'Boolean'})
+        self.assertRaises(TypeError, p.get_value, 5)
+
     def test_list_string(self):
         p = properties.Property({'Type': 'List'})
         self.assertRaises(TypeError, p.get_value, 'foo')
@@ -866,6 +911,21 @@ class PropertyTest(common.HeatTestCase):
         p = properties.Property({'Type': 'List'})
         self.assertRaises(TypeError, p.get_value, {'foo': 'bar'})
 
+    def test_list_is_delimited(self):
+        p = properties.Property({'Type': 'List'})
+        self.assertRaises(TypeError, p.get_value, 'foo,bar')
+        p.schema.allow_conversion = True
+        self.assertEqual(['foo', 'bar'], p.get_value('foo,bar'))
+        self.assertEqual(['foo'], p.get_value('foo'))
+
+    def test_map_path(self):
+        p = properties.Property({'Type': 'Map'}, name='test', path='parent')
+        self.assertEqual('parent.test', p.path)
+
+    def test_list_path(self):
+        p = properties.Property({'Type': 'List'}, name='0', path='parent')
+        self.assertEqual('parent.0', p.path)
+
     def test_list_maxlength_good(self):
         schema = {'Type': 'List',
                   'MaxLength': '3'}
@@ -945,7 +1005,7 @@ class PropertyTest(common.HeatTestCase):
         ex = self.assertRaises(exception.StackValidationFailed,
                                p.get_value, {'valid': 'fish'}, True)
         self.assertEqual('Property error: valid: "fish" is not a '
-                         'valid boolean', six.text_type(ex))
+                         'valid boolean', str(ex))
 
     def test_map_schema_missing_data(self):
         map_schema = {'valid': {'Type': 'Boolean'}}
@@ -958,7 +1018,7 @@ class PropertyTest(common.HeatTestCase):
         ex = self.assertRaises(exception.StackValidationFailed,
                                p.get_value, {}, True)
         self.assertEqual('Property error: Property valid not assigned',
-                         six.text_type(ex))
+                         str(ex))
 
     def test_list_schema_good(self):
         map_schema = {'valid': {'Type': 'Boolean'}}
@@ -977,7 +1037,7 @@ class PropertyTest(common.HeatTestCase):
                                p.get_value,
                                [{'valid': 'True'}, {'valid': 'fish'}], True)
         self.assertEqual('Property error: [1].valid: "fish" is not '
-                         'a valid boolean', six.text_type(ex))
+                         'a valid boolean', str(ex))
 
     def test_list_schema_int_good(self):
         list_schema = {'Type': 'Integer'}
@@ -990,7 +1050,7 @@ class PropertyTest(common.HeatTestCase):
         ex = self.assertRaises(exception.StackValidationFailed,
                                p.get_value, [42, 'fish'], True)
         self.assertEqual("Property error: [1]: Value 'fish' is not "
-                         "an integer", six.text_type(ex))
+                         "an integer", str(ex))
 
 
 class PropertiesTest(common.HeatTestCase):
@@ -1030,6 +1090,14 @@ class PropertiesTest(common.HeatTestCase):
     def test_missing_required(self):
         self.assertRaises(ValueError, self.props.get, 'required_int')
 
+    @mock.patch.object(translation.Translation, 'has_translation')
+    @mock.patch.object(translation.Translation, 'translate')
+    def test_required_with_translate_no_value(self, m_t, m_ht):
+        m_t.return_value = None
+        m_ht.return_value = True
+
+        self.assertRaises(ValueError, self.props.get, 'required_int')
+
     def test_integer_bad(self):
         self.assertRaises(ValueError, self.props.get, 'bad_int')
 
@@ -1050,7 +1118,7 @@ class PropertiesTest(common.HeatTestCase):
         ex = self.assertRaises(KeyError, self.props.get_user_value, 'foo')
         # Note we have to use args here: https://bugs.python.org/issue2651
         self.assertEqual('Invalid Property foo',
-                         six.text_type(ex.args[0]))
+                         str(ex.args[0]))
 
     def test_bad_key(self):
         self.assertEqual('wibble', self.props.get('foo', 'wibble'))
@@ -1059,7 +1127,7 @@ class PropertiesTest(common.HeatTestCase):
         ex = self.assertRaises(KeyError, self.props.__getitem__, 'foo')
         # Note we have to use args here: https://bugs.python.org/issue2651
         self.assertEqual('Invalid Property foo',
-                         six.text_type(ex.args[0]))
+                         str(ex.args[0]))
 
     def test_none_string(self):
         schema = {'foo': {'Type': 'String'}}
@@ -1191,7 +1259,7 @@ class PropertiesTest(common.HeatTestCase):
         # define properties with function and constraint
         props = properties.Properties(
             schema,
-            {'foo': cfn_funcs.ResourceRef(
+            {'foo': hot_funcs.GetResource(
                 stack, 'get_resource', 'another_res')},
             test_resolver)
 
@@ -1288,7 +1356,6 @@ class PropertiesTest(common.HeatTestCase):
             "DBUsername": {
                 "type": "string",
                 "description": "The WordPress database admin account username",
-                "default": "admin",
                 "required": False,
                 'update_allowed': True,
                 'immutable': False,
@@ -1304,7 +1371,6 @@ class PropertiesTest(common.HeatTestCase):
             "LinuxDistribution": {
                 "type": "string",
                 "description": "Distribution of choice",
-                "default": "F17",
                 "required": False,
                 'update_allowed': True,
                 'immutable': False,
@@ -1316,7 +1382,6 @@ class PropertiesTest(common.HeatTestCase):
             "InstanceType": {
                 "type": "string",
                 "description": "WebServer EC2 instance type",
-                "default": "m1.large",
                 "required": False,
                 'update_allowed': True,
                 'immutable': False,
@@ -1337,7 +1402,6 @@ class PropertiesTest(common.HeatTestCase):
             "DBRootPassword": {
                 "type": "string",
                 "description": "Root password for MySQL",
-                "default": "admin",
                 "required": False,
                 'update_allowed': True,
                 'immutable': False,
@@ -1361,7 +1425,6 @@ class PropertiesTest(common.HeatTestCase):
             "DBPassword": {
                 "type": "string",
                 "description": "The WordPress database admin account password",
-                "default": "admin",
                 "required": False,
                 'update_allowed': True,
                 'immutable': False,
@@ -1377,7 +1440,6 @@ class PropertiesTest(common.HeatTestCase):
             "DBName": {
                 "type": "string",
                 "description": "The WordPress database name",
-                "default": "wordpress",
                 "required": False,
                 'update_allowed': True,
                 'immutable': False,
@@ -1491,7 +1553,6 @@ class PropertiesTest(common.HeatTestCase):
             "InstanceType": {
                 "type": "string",
                 "description": "WebServer EC2 instance type",
-                "default": "m1.large",
                 "required": False,
                 'update_allowed': True,
                 'immutable': False,
@@ -1504,7 +1565,6 @@ class PropertiesTest(common.HeatTestCase):
                 ]
             },
             "LinuxDistribution": {
-                "default": "F17",
                 "type": "string",
                 "description": "Distribution of choice",
                 "required": False,
@@ -1519,7 +1579,6 @@ class PropertiesTest(common.HeatTestCase):
             "DBName": {
                 "type": "string",
                 "description": "The WordPress database name",
-                "default": "wordpress",
                 "required": False,
                 'update_allowed': True,
                 'immutable': False,
@@ -1534,7 +1593,6 @@ class PropertiesTest(common.HeatTestCase):
             "DBUsername": {
                 "type": "string",
                 "description": "The WordPress database admin account username",
-                "default": "admin",
                 "required": False,
                 'update_allowed': True,
                 'immutable': False,
@@ -1549,7 +1607,6 @@ class PropertiesTest(common.HeatTestCase):
             "DBPassword": {
                 "type": "string",
                 "description": "The WordPress database admin account password",
-                "default": "admin",
                 "required": False,
                 'update_allowed': True,
                 'immutable': False,
@@ -1564,7 +1621,6 @@ class PropertiesTest(common.HeatTestCase):
             "DBRootPassword": {
                 "type": "string",
                 "description": "Root password for MySQL",
-                "default": "admin",
                 "required": False,
                 'update_allowed': True,
                 'immutable': False,
@@ -1596,6 +1652,30 @@ class PropertiesTest(common.HeatTestCase):
         props_b = properties.Properties(schema, {'foo': 1})
         self.assertTrue(props_a != props_b)
 
+    def test_description_substitution(self):
+        schema = {
+            'description': properties.Schema('String',
+                                             update_allowed=True),
+            'not_description': properties.Schema('String',
+                                                 update_allowed=True),
+        }
+        blank_rsrc = rsrc_defn.ResourceDefinition('foo', 'FooResource', {},
+                                                  description='Foo resource')
+        bar_rsrc = rsrc_defn.ResourceDefinition('foo', 'FooResource',
+                                                {'description': 'bar'},
+                                                description='Foo resource')
+
+        blank_props = blank_rsrc.properties(schema)
+        self.assertEqual('Foo resource', blank_props['description'])
+        self.assertEqual(None, blank_props['not_description'])
+
+        replace_schema = {'description': properties.Schema('String')}
+        empty_props = blank_rsrc.properties(replace_schema)
+        self.assertEqual(None, empty_props['description'])
+
+        bar_props = bar_rsrc.properties(schema)
+        self.assertEqual('bar', bar_props['description'])
+
 
 class PropertiesValidationTest(common.HeatTestCase):
     def test_required(self):
@@ -1632,15 +1712,15 @@ class PropertiesValidationTest(common.HeatTestCase):
         schema = {'foo': {'Type': 'String'}}
         props = properties.Properties(schema, {'foo': ['foo', 'bar']})
         ex = self.assertRaises(exception.StackValidationFailed, props.validate)
-        self.assertEqual('Property error: foo: Value must be a string',
-                         six.text_type(ex))
+        self.assertIn('Property error: foo: Value must be a string',
+                      str(ex))
 
     def test_dict_instead_string(self):
         schema = {'foo': {'Type': 'String'}}
         props = properties.Properties(schema, {'foo': {'foo': 'bar'}})
         ex = self.assertRaises(exception.StackValidationFailed, props.validate)
-        self.assertEqual('Property error: foo: Value must be a string',
-                         six.text_type(ex))
+        self.assertIn('Property error: foo: Value must be a string',
+                      str(ex))
 
     def test_none_string(self):
         schema = {'foo': {'Type': 'String'}}
@@ -1807,7 +1887,7 @@ class PropertiesValidationTest(common.HeatTestCase):
         ex = self.assertRaises(exception.StackValidationFailed,
                                props.validate)
         self.assertEqual('Property error: foo[0]: Unknown Property bar',
-                         six.text_type(ex))
+                         str(ex))
 
     def test_nested_properties_schema_invalid_property_in_map(self):
         child_schema = {'Key': {'Type': 'String',
@@ -1826,7 +1906,7 @@ class PropertiesValidationTest(common.HeatTestCase):
         ex = self.assertRaises(exception.StackValidationFailed,
                                props.validate)
         self.assertEqual('Property error: foo.boo: Unknown Property bar',
-                         six.text_type(ex))
+                         str(ex))
 
     def test_more_nested_properties_schema_invalid_property_in_list(self):
         nested_child_schema = {'Key': {'Type': 'String',
@@ -1844,7 +1924,7 @@ class PropertiesValidationTest(common.HeatTestCase):
         ex = self.assertRaises(exception.StackValidationFailed,
                                props.validate)
         self.assertEqual('Property error: foo[0].doo: Unknown Property bar',
-                         six.text_type(ex))
+                         str(ex))
 
     def test_more_nested_properties_schema_invalid_property_in_map(self):
         nested_child_schema = {'Key': {'Type': 'String',
@@ -1862,7 +1942,7 @@ class PropertiesValidationTest(common.HeatTestCase):
         ex = self.assertRaises(exception.StackValidationFailed,
                                props.validate)
         self.assertEqual('Property error: foo.boo.doo: Unknown Property bar',
-                         six.text_type(ex))
+                         str(ex))
 
     def test_schema_to_template_empty_schema(self):
         schema = {}
@@ -1874,9 +1954,8 @@ class PropertiesValidationTest(common.HeatTestCase):
         self.assertEqual({}, props)
 
     def test_update_allowed_and_immutable_contradict(self):
-        schema = {'foo': properties.Schema(
-            properties.Schema.STRING,
-            update_allowed=True,
-            immutable=True)}
-        props = properties.Properties(schema, {})
-        self.assertRaises(exception.InvalidSchemaError, props.validate)
+        self.assertRaises(exception.InvalidSchemaError,
+                          properties.Schema,
+                          properties.Schema.STRING,
+                          update_allowed=True,
+                          immutable=True)
