@@ -14,7 +14,6 @@ import copy
 import json
 
 from heatclient import exc
-import six
 import yaml
 
 from heat_integrationtests.functional import functional_base
@@ -42,9 +41,6 @@ outputs:
     value: {get_attr: [random_group, value]}
 '''
 
-    def setUp(self):
-        super(ResourceGroupTest, self).setUp()
-
     def test_resource_group_zero_novalidate(self):
         # Nested resources should be validated only when size > 0
         # This allows features to be disabled via size=0 without
@@ -85,16 +81,18 @@ resources:
 
         # Prove validation works for non-zero create/update
         template_two_nested = self.template.replace("count: 0", "count: 2")
-        expected_err = "Value 'BAD' is not an integer"
+        expected_err = ("resources.random_group<nested_stack>.resources."
+                        "0<provider.yaml>.resources.random: : "
+                        "Value 'BAD' is not an integer")
         ex = self.assertRaises(exc.HTTPBadRequest, self.update_stack,
                                stack_identifier, template_two_nested,
                                environment=env, files=files)
-        self.assertIn(expected_err, six.text_type(ex))
+        self.assertIn(expected_err, str(ex))
 
         ex = self.assertRaises(exc.HTTPBadRequest, self.stack_create,
                                template=template_two_nested,
                                environment=env, files=files)
-        self.assertIn(expected_err, six.text_type(ex))
+        self.assertIn(expected_err, str(ex))
 
     def _validate_resources(self, stack_identifier, expected_count):
         resources = self.list_group_resources(stack_identifier,
@@ -323,6 +321,37 @@ outputs:
         updated_rand = self._stack_output(stack1, 'random1')
         self.assertNotEqual(initial_rand, updated_rand)
 
+    def test_validation(self):
+        resource_group = '''
+heat_template_version: 2016-10-14
+
+parameters:
+  the_count:
+    type: number
+
+resources:
+
+  the_group:
+    type: OS::Heat::ResourceGroup
+    properties:
+      count: {get_param: the_count}
+      resource_def:
+        type: OS::Heat::RandomString
+'''
+        ret = self.client.stacks.validate(template=resource_group)
+        expected = {'Description': 'No description',
+                    'Environment': {'event_sinks': [],
+                                    'parameter_defaults': {},
+                                    'parameters': {},
+                                    'resource_registry': {u'resources': {}}},
+                    'Parameters': {
+                        'the_count': {'Description': '',
+                                      'Label': 'the_count',
+                                      'NoEcho': 'false',
+                                      'Type': 'Number'}}}
+
+        self.assertEqual(expected, ret)
+
 
 class ResourceGroupTestNullParams(functional_base.FunctionalTestsBase):
     template = '''
@@ -377,9 +406,6 @@ outputs:
         )),
     ]
 
-    def setUp(self):
-        super(ResourceGroupTestNullParams, self).setUp()
-
     def test_create_pass_zero_parameter(self):
         templ = self.template.replace('type: empty',
                                       'type: %s' % self.p_type)
@@ -417,9 +443,6 @@ outputs:
     value: {get_attr: [group1, resource.1.value]}
 '''
 
-    def setUp(self):
-        super(ResourceGroupAdoptTest, self).setUp()
-
     def _yaml_to_json(self, yaml_templ):
         return yaml.safe_load(yaml_templ)
 
@@ -489,9 +512,6 @@ resources:
       wait_secs: 2
 '''
 
-    def setUp(self):
-        super(ResourceGroupErrorResourceTest, self).setUp()
-
     def test_fail(self):
         stack_identifier = self.stack_create(
             template=self.template,
@@ -591,6 +611,36 @@ resources:
                                    created=10,
                                    deleted=10)
 
+    def test_resource_group_update_replace_template_changed(self):
+        """Test rolling update(replace)with child template path changed.
+
+        Simple rolling update replace with child template path changed.
+        """
+
+        nested_templ = '''
+heat_template_version: "2013-05-23"
+resources:
+  oops:
+    type: OS::Heat::TestResource
+'''
+
+        create_template = yaml.safe_load(copy.deepcopy(self.template))
+        grp = create_template['resources']['random_group']
+        grp['properties']['resource_def'] = {'type': '/opt/provider.yaml'}
+        files = {'/opt/provider.yaml': nested_templ}
+
+        policy = grp['update_policy']['rolling_update']
+        policy['min_in_service'] = '1'
+        policy['max_batch_size'] = '3'
+        stack_identifier = self.stack_create(template=create_template,
+                                             files=files)
+        update_template = create_template.copy()
+        grp = update_template['resources']['random_group']
+        grp['properties']['resource_def'] = {'type': '/opt1/provider.yaml'}
+        files = {'/opt1/provider.yaml': nested_templ}
+
+        self.update_stack(stack_identifier, update_template, files=files)
+
     def test_resource_group_update_scaledown(self):
         """Test rolling update with scaledown.
 
